@@ -27,10 +27,17 @@ class SQLiteYStore(LoggingConfigurable, _SQLiteYStore, metaclass=SQLiteYStoreMet
         directory.""",
     )
 
-    document_ttl = Int(
+    squash_after_inactivity_of = Int(
         None,
         allow_none=True,
         config=True,
         help="""The document time-to-live in seconds. Defaults to None (document history is never
         cleared).""",
     )
+    document_ttl = Int(
+        None,
+        allow_none=True,
+        config=True,
+        help="""The document time-to-live in seconds. Deprecated in favor of 'squash_after_inactivity_of'.
+        Defaults to None (document history is never cleared).""",
+    )
